package java.security.cert;

import java.security.InvalidKeyException;
import java.security.NoSuchAlgorithmException;
import java.security.SignatureException;
import java.security.InvalidAlgorithmParameterException;
import java.security.ProviderException;
import sun.security.util.SignatureUtil;
import java.security.Signature;
import java.security.Provider;
import java.security.PublicKey;
import java.util.Collection;
import java.util.List;
import sun.security.x509.X509CertImpl;
import java.security.Principal;
import java.math.BigInteger;
import java.util.Date;
import javax.security.auth.x500.X500Principal;

public abstract class X509Certificate extends Certificate implements X509Extension
{
    private static final long serialVersionUID = -2491127588187038216L;
    private transient X500Principal subjectX500Principal;
    private transient X500Principal issuerX500Principal;
    
    protected X509Certificate() {
        super("X.509");
    }
    
    public abstract void checkValidity() throws CertificateExpiredException, CertificateNotYetValidException;
    
    public abstract void checkValidity(Date p0) throws CertificateExpiredException, CertificateNotYetValidException;
    
    public abstract int getVersion();
    
    public abstract BigInteger getSerialNumber();
    
    public abstract Principal getIssuerDN();
    
    public X500Principal getIssuerX500Principal() {
        if (this.issuerX500Principal == null) {
            this.issuerX500Principal = X509CertImpl.getIssuerX500Principal(this);
        }
        return this.issuerX500Principal;
    }
    
    public abstract Principal getSubjectDN();
    
    public X500Principal getSubjectX500Principal() {
        if (this.subjectX500Principal == null) {
            this.subjectX500Principal = X509CertImpl.getSubjectX500Principal(this);
        }
        return this.subjectX500Principal;
    }
    
    public abstract Date getNotBefore();
    
    public abstract Date getNotAfter();
    
    public abstract byte[] getTBSCertificate() throws CertificateEncodingException;
    
    public abstract byte[] getSignature();
    
    public abstract String getSigAlgName();
    
    public abstract String getSigAlgOID();
    
    public abstract byte[] getSigAlgParams();
    
    public abstract boolean[] getIssuerUniqueID();
    
    public abstract boolean[] getSubjectUniqueID();
    
    public abstract boolean[] getKeyUsage();
    
    public List<String> getExtendedKeyUsage() throws CertificateParsingException {
        return X509CertImpl.getExtendedKeyUsage(this);
    }
    
    public abstract int getBasicConstraints();
    
    public Collection<List<?>> getSubjectAlternativeNames() throws CertificateParsingException {
        return X509CertImpl.getSubjectAlternativeNames(this);
    }
    
    public Collection<List<?>> getIssuerAlternativeNames() throws CertificateParsingException {
        return X509CertImpl.getIssuerAlternativeNames(this);
    }
    
    @Override
    public void verify(PublicKey key, Provider provider) throws CertificateException, NoSuchAlgorithmException, InvalidKeyException, SignatureException {
        String sigAlgName = this.getSigAlgName();
        Signature s = (provider == null) ? Signature.getInstance(sigAlgName) : Signature.getInstance(sigAlgName, provider);
        try {
            SignatureUtil.initVerifyWithParam(s, key, SignatureUtil.getParamSpec(sigAlgName, this.getSigAlgParams()));
        }
        catch (ProviderException ex) {
            throw new CertificateException(ex.getMessage(), ex.getCause());
        }
        catch (InvalidAlgorithmParameterException ex2) {
            throw new CertificateException(ex2);
        }
        byte[] tbsCertificate = this.getTBSCertificate();
        s.update(tbsCertificate, 0, tbsCertificate.length);
        if (!s.verify(this.getSignature())) {
            throw new SignatureException("Signature does not match.");
        }
        return;
    }
}
